# DJAqua
#    description:   A plugin to announce a track's artist and title before and after play
#    author:        Richard Schwarting (aquarichy@gmail.com)
#    license:       GPL2
#    (out of) date: 2009/04/06

# Copyright (C) 2009 Richard Schwarting
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import rhythmdb
import rb
import sys
import glib
import time

from DJSpeaker   import DJSpeaker
from DJException import DJException

class DJAquaPlugin(rb.Plugin):
    def __init__(self):
        rb.Plugin.__init__(self)
        self.fout = open("/tmp/djaqua.out", 'w')
        self.debug_on = False
        
    def activate(self, shell):
        try:
            self.dj = DJSpeaker("Welcome to radio rhythm box")
        except DJException, e:
            sys.stderr.write("Rhythmbox:DJAqua: Failed to setup voice: %s\n" % str(e))
            self.debug(str(e))
            return
            
        self.db = shell.get_property("db")
        self.sp = shell.get_player()
        #self.old_volume = self.sp.get_volume()

        self.sp.connect('playing-song-changed', self.playing_song_changed)
        self.cur_entry = None

    def create_script_for_entry(self, entry, prefix, default):
        if entry is not None:
            title  = self.db.entry_get(entry, rhythmdb.PROP_TITLE)
            artist = self.db.entry_get(entry, rhythmdb.PROP_ARTIST)
            if title is not None or artist is not None:
                script = prefix + " "
                if title is not None:
                    script = script + title + " "
                if artist is not None:
                    script = script + ", by,  " + artist
            else:
                script = default
        
            return script

    def debug(self, message):
        if self.debug_on and self.dj.is_working():
            sys.stderr.write(message + "\n")
            print >> self.fout, message
            self.fout.flush()
        
#     def dj_done_callback(self, callback_type=None, event_types=None):
#         self.debug("dj_done_callback: entering, with callback_type: %s" % str(callback_type))
#         self.debug("dj_done_callback: sadly can't rely on CallbackType.END or CANCEL to be called; race in SpeechDispatcher");
#         return # todo: check status of rh bugs 494298, 587285 upstream, see if we can rely on this

#         if callback_type == speechd.CallbackType.BEGIN:
#             # This callback will only occasionally be called for this event
#             # there is a bug in Speech Dispatcher,
#             # see https://bugzilla.redhat.com/show_bug.cgi?id=494298 
#             # see https://bugzilla.redhat.com/show_bug.cgi?id=587285 
#             self.lower_volume();
#         elif callback_type in (speechd.CallbackType.END, speechd.CallbackType.CANCEL):
#             self.raise_volume();

    def playing_song_changed(self, sp, entry):
        # todo: consider cancelling existing ones
        # todo: detect when we're closing, so we don't needless lower the volume without remaining alive long enough to return it
        self.debug("playing_song_changed: entering")
        
        self.debug("playing_song_changed: creating scripts to read out")
        script = ""
        if self.cur_entry is not None:
            script = script + " " + self.create_script_for_entry(self.cur_entry, "That was, ", "Who knows what that was.")

        self.cur_entry = entry

        if self.cur_entry is not None:
            script = script + " " + self.create_script_for_entry(self.cur_entry, "This is, ", "Who knows what comes next.")

        if script is not None:
            self.debug("playing_song_changed: have script, speaking!>")

            if self.cur_entry is not None:
                # only play with volume if we're playing something new at this same time
                self.lower_volume();
                glib.timeout_add_seconds(len(script) / 15, self.raise_volume); # "scientifically" determined 15 chars/sec
            self.dj.speak(script);

            # Until rh bugs 494298,587285  are fixed upstream, we cannot rely on
            # the CANCEL/END callbacks, so we are going to go on faith
            # that this will actually play, and will lower the volume
            # of the music now so we can hear the radio host; 
            # a sample of 11 speeches revealed that on average, it took 1 second
            # for 15 characters; not as nice as working callbacks; oh well
#             self.dj.speak(script,
#                           callback=self.dj_done_callback,
#                           event_types=(speechd.CallbackType.BEGIN,
#                                        speechd.CallbackType.CANCEL,
#                                        speechd.CallbackType.END))
            
            self.debug("playing_song_changed: <script spoken '%s'" % script)
        else:
            self.debug("playing_song_changed: have script not")
        
        self.debug("playing_song_changed: done")

    def deactive(self, shell):
        # TODO: figure out how to properly deactivate :) 
        self.client.close()

    # This should usually be used to quiet down the music while DJSpeaker is announcing
    # the track's artist and title
    def lower_volume(self):
        self.debug("dj_done_callback(BEGIN): (current volume): %f" % self.sp.get_volume())
        self.sp.set_volume(self.sp.get_volume() / 3)
        self.debug("dj_done_callback(BEGIN):     (new volume): %f" % self.sp.get_volume())
        return False # for the benefit of glib.timeout_add_seconds()

    # This is used to restore the audio after it was earlier lowered.
    def raise_volume(self):
        self.debug("dj_done_callback(END/CANCEL): (current volume): %f" % self.sp.get_volume())
        self.sp.set_volume(self.sp.get_volume() * 3)
        self.debug("dj_done_callback(END/CANCEL):     (new volume): %f" % self.sp.get_volume())
        return False # for the benefit of glib.timeout_add_seconds()

        

        
        


        
