# DJAqua
#    description:   A plugin to announce a track's artist and title before and after play
#    author:        Richard Schwarting (aquarichy@gmail.com)
#    license:       GPL2
#    (out of) date: 2009/04/06

# Copyright (C) 2009 Richard Schwarting
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import socket
import speechd
import glib
from DJException import DJException

# DJSpeaker just wraps speechd and eventually gnome-speech, preferring speechd's API # note: gave up on gnome-speech for now, since KillBonobo
class DJSpeaker:
    def __init__(self, welcome_msg):
        try:
            self.dj = speechd.SSIPClient('djaqua')
        except socket.error, e:
            self.dj = None
            raise DJException("Rhythmbox:DJAqua: Failed to instantiate Speech Dispatcher: %s.  Are you sure it is running?" % str(e))

        try:
            output_module = self._select_output_module()
        except DJException, e:
            # pass it up, explicitly
            raise e

        self.dj.set_output_module(output_module)
        self.dj.set_volume(100)
        self.dj.set_priority(speechd.client.Priority.IMPORTANT)

        self._test_callbacks_with_welcome(welcome_msg)
        # TODO: add UI to let a user select their language, voices
        # self.dj.set_language('en')
        
        # for voice in self.dj.list_synthesis_voices():
        #    cat.set_synthesis_voice(voice[0])

    def _test_callbacks_with_welcome(self, welcome_msg):
        def _test_callback(callback_type):
            # confirm callbacks, and probably speech dispatcher, are working
            self.callbacks_work = True

        def _check_success():
            if not self.callbacks_work:
                msg = "Rhythmbox:DJAqua:DJSpeaker: Speech may not be working." +\
                    "    Check whether Speech Dispatcher had any issues connecting to pulseaudio." +\
                    "    Check Speech Dispatcher's logs (probably /var/log/speech-dispatcher/)." +\
                    "    (note: on Fedora 10, the directory by default was not created and had to be manually.)"
            else:
                msg = "Rhythmbox:DJAqua:DJSpeaker: Callbacks seem to work.  Voice should play if not muted.\n"
                
            sys.stdout.write(msg);
            sys.stderr.write(msg);
                
            return False

        msg = "Rhythmbox:DJAqua:DJSpeaker: testing speech functionality."
        sys.stdout.write(msg)
        sys.stderr.write(msg)

        self.callbacks_work = False
        self.dj.set_volume(100)
        self.speak(welcome_msg, _test_callback)

        glib.timeout_add_seconds(5, _check_success);

    # selects an output module (e.g. festival, flite, espeak, etc) 
    def _select_output_module(self):
        output_modules = self.dj.list_output_modules()
        if len(output_modules) <= 0:
            raise DJException("Rhythmbox:DJAqua: Speech Dispatcher provides no output modules :(\n")
        if 'festival' in output_modules:
            output_module = 'festival'
        else:
            sys.stderr.write("Rhythmbox:DJAqua: 'festival' output module not found.  Trying flite.\n")
            if 'flite' in output_modules:
                output_module = 'flite'
            else:
                output_module = output_modules[0];
                sys.stderr.write("Rhythmbox:DJAqua: 'flite' output module not found.  Using first from those known: %s\n" % output_module)
        return output_module

    def speak(self, text, callback=None, event_types=None):
        self.dj.speak(text, callback=callback, event_types=event_types)
        
    def cancel(self):
        self.dj.cancel()
    
    def is_working(self):
        return self.callbacks_work
        


